@@ -25,6 +25,7 @@ import jxl.format.PageOrientation;
 import jxl.format.PaperSize;
 import jxl.biff.SheetRangeImpl;
 import jxl.Range;
+import jxl.format.PageOrder;
 
 /**
  * This is a bean which client applications may use to get/set various
@@ -37,6 +38,11 @@ public final class SheetSettings
    * The page orientation
    */
   private PageOrientation orientation;
+  
+  /**
+   * The page order
+   */
+  private PageOrder pageOrder;
 
   /**
    * The paper size for printing
@@ -237,6 +243,16 @@ public final class SheetSettings
    */
   private Range printArea;
 
+  /**
+   * The print row titles
+   */
+  private Range printTitlesRow;
+
+  /**
+   * The print column titles
+   */
+  private Range printTitlesCol;
+    
   /**
    * A handle to the sheet - used internally for ranges
    */
@@ -247,6 +263,8 @@ public final class SheetSettings
   // **
   private static final PageOrientation DEFAULT_ORIENTATION =
     PageOrientation.PORTRAIT;
+  private static final PageOrder DEFAULT_ORDER =
+    PageOrder.RIGHT_THEN_DOWN;
   private static final PaperSize DEFAULT_PAPER_SIZE = PaperSize.A4;
   private static final double DEFAULT_HEADER_MARGIN = 0.5;
   private static final double DEFAULT_FOOTER_MARGIN = 0.5;
@@ -272,6 +290,7 @@ public final class SheetSettings
   {
     sheet = s; // for internal use, when accessing ranges
     orientation        = DEFAULT_ORIENTATION;
+    pageOrder          = DEFAULT_ORDER;
     paperSize          = DEFAULT_PAPER_SIZE;
     sheetProtected     = false;
     hidden             = false;
@@ -314,6 +333,7 @@ public final class SheetSettings
 
     sheet = s;  // for internal use when accessing ranges
     orientation    = copy.orientation;
+    pageOrder      = copy.pageOrder;
     paperSize      = copy.paperSize;
     sheetProtected = copy.sheetProtected;
     hidden         = copy.hidden;
@@ -360,6 +380,26 @@ public final class SheetSettings
          copy.getPrintArea().getBottomRight().getColumn(),
          copy.getPrintArea().getBottomRight().getRow());
     }
+    
+    if (copy.printTitlesRow != null)
+    {
+      printTitlesRow = new SheetRangeImpl
+        (sheet,
+         copy.getPrintTitlesRow().getTopLeft().getColumn(),
+         copy.getPrintTitlesRow().getTopLeft().getRow(),
+         copy.getPrintTitlesRow().getBottomRight().getColumn(),
+         copy.getPrintTitlesRow().getBottomRight().getRow());
+    }    
+
+    if (copy.printTitlesCol != null)
+    {
+      printTitlesCol = new SheetRangeImpl
+        (sheet,
+         copy.getPrintTitlesCol().getTopLeft().getColumn(),
+         copy.getPrintTitlesCol().getTopLeft().getRow(),
+         copy.getPrintTitlesCol().getBottomRight().getColumn(),
+         copy.getPrintTitlesCol().getBottomRight().getRow());
+    }    
   }
 
   /**
@@ -382,6 +422,26 @@ public final class SheetSettings
     return orientation;
   }
 
+  /**
+   * Accessor for the order
+   * 
+   * @return
+   */
+  public PageOrder getPageOrder() 
+  {
+    return pageOrder;
+  }
+
+  /**
+   * Sets the page order for printing this sheet
+   * 
+   * @param order
+   */
+  public void setPageOrder(PageOrder order) 
+  {
+    this.pageOrder = order;
+  }
+
   /**
    * Sets the paper size to be used when printing this sheet
    *
@@ -1218,4 +1278,67 @@ public final class SheetSettings
     return printArea;
   }
 
+  /**
+   * Sets both of the print titles for this sheet
+   *
+   * @param firstRow the first row of the print row titles
+   * @param lastRow the last row of the print row titles
+   * @param firstCol the first column of the print column titles
+   * @param lastCol the last column of the print column titles
+   */
+  public void setPrintTitles(int firstRow, 
+                             int lastRow,  
+                             int firstCol,
+                             int lastCol)
+  {
+    setPrintTitlesRow(firstRow, lastRow);
+    setPrintTitlesCol(firstCol, lastCol);
+  }
+
+  /**
+   * Sets the print row titles for this sheet
+   *
+   * @param firstRow the first row of the print titles
+   * @param lastRow the last row of the print titles
+   */
+  public void setPrintTitlesRow(int firstRow, 
+                           		int lastRow)
+  {
+    printTitlesRow = new SheetRangeImpl(sheet, 0, firstRow, 
+    									     255, lastRow);
+  }
+  
+  /**
+   * Sets the print column titles for this sheet
+   *
+   * @param firstRow the first row of the print titles
+   * @param lastRow the last row of the print titles
+   */
+  public void setPrintTitlesCol(int firstCol, 
+                           		int lastCol)
+  {
+    printTitlesCol = new SheetRangeImpl(sheet, firstCol, 0, 
+                                      		    lastCol, 65535);
+  }
+  
+  /**
+   * Accessor for the print row titles
+   *
+   * @return the print row titles, or NULL if one is not defined for this sheet
+   */
+  public Range getPrintTitlesRow()
+  {
+    return printTitlesRow;
+  }
+  
+  /**
+   * Accessor for the print column titles
+   *
+   * @return the print column titles, or NULL if one is not defined for this 
+   * sheet
+   */
+  public Range getPrintTitlesCol()
+  {
+    return printTitlesCol;
+  }
 }
