@@ -221,6 +221,16 @@ public class SheetImpl implements Sheet
    */
   private int[] columnBreaks;
 
+  /**
+   * The maximum row outline level
+   */
+  private int maxRowOutlineLevel;
+
+  /**
+   * The maximum column outline level
+   */
+  private int maxColumnOutlineLevel;
+
   /**
    * The list of local names for this sheet
    */
@@ -695,6 +705,8 @@ public class SheetImpl implements Sheet
     workspaceOptions = reader.getWorkspaceOptions();
     plsRecord = reader.getPLS();
     buttonPropertySet = reader.getButtonPropertySet();
+    maxRowOutlineLevel = reader.getMaxRowOutlineLevel();
+    maxColumnOutlineLevel = reader.getMaxColumnOutlineLevel();
 
     reader = null;
 
@@ -722,11 +734,36 @@ public class SheetImpl implements Sheet
         NameRecord nr = (NameRecord) it.next();
         if (nr.getBuiltInName() == BuiltInName.PRINT_AREA)
         {
-          NameRecord.NameRange rng = nr.getRanges()[0];
-          settings.setPrintArea(rng.getFirstColumn(),
-                                rng.getFirstRow(),
-                                rng.getLastColumn(),
-                                rng.getLastRow());
+          if(nr.getRanges().length > 0)
+          {
+            NameRecord.NameRange rng = nr.getRanges()[0];
+            settings.setPrintArea(rng.getFirstColumn(),
+                                  rng.getFirstRow(),
+                                  rng.getLastColumn(),
+                                  rng.getLastRow());
+          }
+        }
+        else if (nr.getBuiltInName() == BuiltInName.PRINT_TITLES)
+       	{
+          // There can be 1 or 2 entries.  
+          // Row entries have hardwired column entries (first and last
+          //  possible column)
+          // Column entries have hardwired row entries (first and last 
+          // possible row)
+          for (int i = 0 ; i < nr.getRanges().length ; i++)
+          {
+            NameRecord.NameRange rng = nr.getRanges()[i];
+            if (rng.getFirstColumn() == 0 && rng.getLastColumn() == 255)
+            {
+              settings.setPrintTitlesRow(rng.getFirstRow(),
+                                         rng.getLastRow());
+            }
+            else
+            {
+              settings.setPrintTitlesCol(rng.getFirstColumn(),
+                                         rng.getLastColumn());
+            }
+          }
         }
       }
     }
@@ -1162,4 +1199,25 @@ public class SheetImpl implements Sheet
   {
     return autoFilter;
   }
+
+  /** 
+   * Accessor for the maximum column outline level.  Used during a copy
+   *
+   * @return the maximum column outline level, or 0 if no outlines/groups
+   */
+  public int getMaxColumnOutlineLevel() 
+  {
+    return maxColumnOutlineLevel;
+  }
+
+  /** 
+   * Accessor for the maximum row outline level.  Used during a copy
+   *
+   * @return the maximum row outline level, or 0 if no outlines/groups
+   */
+  public int getMaxRowOutlineLevel() 
+  {
+    return maxRowOutlineLevel;
+  }
+
 }
