@@ -62,6 +62,7 @@ import jxl.biff.drawing.NoteRecord;
 import jxl.biff.drawing.ObjRecord;
 import jxl.biff.drawing.TextObjectRecord;
 import jxl.biff.formula.FormulaException;
+import jxl.format.PageOrder;
 import jxl.format.PageOrientation;
 import jxl.format.PaperSize;
 
@@ -213,6 +214,16 @@ final class SheetReader
    */
   private int[] columnBreaks;
 
+  /**
+   * The maximum row outline level
+   */
+  private int maxRowOutlineLevel;
+
+  /**
+   * The maximum column outline level
+   */
+  private int maxColumnOutlineLevel;
+
   /**
    * The sheet settings
    */
@@ -333,7 +344,7 @@ final class SheetReader
     boolean firstMsoRecord = true;
     
     // Handle to the last conditional format record
-    ConditionalFormatRangeRecord condFormat = null;
+    ConditionalFormat condFormat = null;
 
     // Handle to the autofilter records
     FilterModeRecord filterMode = null;
@@ -355,6 +366,9 @@ final class SheetReader
     // A list of object ids - used for cross referencing
     ArrayList objectIds = new ArrayList();
 
+    // A handle to a continue record read in
+    ContinueRecord continueRecord = null;
+
     while (cont)
     {
       r = excelFile.next();
@@ -549,7 +563,8 @@ final class SheetReader
       }
       else if (type == Type.CONTINUE)
       {
-        ;
+        // don't know what this is for, but keep hold of it anyway
+        continueRecord = new ContinueRecord(r);
       }
       else if (type == Type.NOTE)
       {
@@ -719,7 +734,8 @@ final class SheetReader
         if (!rr.isDefaultHeight() ||
             !rr.matchesDefaultFontHeight() ||
             rr.isCollapsed() ||
-            rr.hasDefaultFormat())
+            rr.hasDefaultFormat() || 
+            rr.getOutlineLevel() != 0)
         {
           rowProperties.add(rr);
         }
@@ -799,29 +815,43 @@ final class SheetReader
       else if (type == Type.SETUP)
       {
         SetupRecord sr = new SetupRecord(r);
-        if (sr.isPortrait())
+        
+        // If the setup record has its not initialized bit set, then
+        // use the sheet settings default values
+        if (sr.getInitialized())
         {
-          settings.setOrientation(PageOrientation.PORTRAIT);
-        }
-        else
-        {
-          settings.setOrientation(PageOrientation.LANDSCAPE);
-        }
-        settings.setPaperSize(PaperSize.getPaperSize(sr.getPaperSize()));
-        settings.setHeaderMargin(sr.getHeaderMargin());
-        settings.setFooterMargin(sr.getFooterMargin());
-        settings.setScaleFactor(sr.getScaleFactor());
-        settings.setPageStart(sr.getPageStart());
-        settings.setFitWidth(sr.getFitWidth());
-        settings.setFitHeight(sr.getFitHeight());
-        settings.setHorizontalPrintResolution
-          (sr.getHorizontalPrintResolution());
-        settings.setVerticalPrintResolution(sr.getVerticalPrintResolution());
-        settings.setCopies(sr.getCopies());
+          if (sr.isPortrait())
+          {
+            settings.setOrientation(PageOrientation.PORTRAIT);
+          }
+          else
+          {
+            settings.setOrientation(PageOrientation.LANDSCAPE);
+          }
+          if (sr.isRightDown())
+          {
+              settings.setPageOrder(PageOrder.RIGHT_THEN_DOWN);
+          }
+          else
+          {
+              settings.setPageOrder(PageOrder.DOWN_THEN_RIGHT);
+          }
+          settings.setPaperSize(PaperSize.getPaperSize(sr.getPaperSize()));
+          settings.setHeaderMargin(sr.getHeaderMargin());
+          settings.setFooterMargin(sr.getFooterMargin());
+          settings.setScaleFactor(sr.getScaleFactor());
+          settings.setPageStart(sr.getPageStart());
+          settings.setFitWidth(sr.getFitWidth());
+          settings.setFitHeight(sr.getFitHeight());
+          settings.setHorizontalPrintResolution
+            (sr.getHorizontalPrintResolution());
+          settings.setVerticalPrintResolution(sr.getVerticalPrintResolution());
+          settings.setCopies(sr.getCopies());
 
-        if (workspaceOptions != null)
-        {
-          settings.setFitToPages(workspaceOptions.getFitToPages());
+          if (workspaceOptions != null)
+          {
+            settings.setFitToPages(workspaceOptions.getFitToPages());
+          }
         }
       }
       else if (type == Type.WSBOOL)
@@ -843,14 +873,15 @@ final class SheetReader
       }
       else if (type == Type.CONDFMT)
       {
-        condFormat = new ConditionalFormatRangeRecord(r);
+        ConditionalFormatRangeRecord cfrr = 
+          new ConditionalFormatRangeRecord(r);
+        condFormat = new ConditionalFormat(cfrr);
+        conditionalFormats.add(condFormat);
       }
       else if (type == Type.CF)
       {
         ConditionalFormatRecord cfr = new ConditionalFormatRecord(r);
-        ConditionalFormat cf = new ConditionalFormat(condFormat, cfr);
-        conditionalFormats.add(cf);
-        condFormat = null;
+        condFormat.addCondition(cfr);
       }
       else if (type == Type.FILTERMODE)
       {
@@ -1009,6 +1040,15 @@ final class SheetReader
 
         if (!workbookSettings.getDrawingsDisabled())
         {
+          // sometimes excel writes out continue records instead of drawing
+          // records, so forcibly hack the stashed continue record into
+          // a drawing record
+          if (msoRecord == null && continueRecord != null)
+          {
+            logger.warn("Cannot find drawing record - using continue record");
+            msoRecord = new MsoDrawingRecord(continueRecord.getRecord());
+            continueRecord = null;
+          }
           handleObjectRecord(objRecord, msoRecord, comments);
           objectIds.add(new Integer(objRecord.getObjectId()));
         }
@@ -1053,6 +1093,14 @@ final class SheetReader
         SaveRecalcRecord cmr = new SaveRecalcRecord(r);
         settings.setRecalculateFormulasBeforeSave(cmr.getRecalculateOnSave());
       }
+      else if (type == Type.GUTS)
+      {
+        GuttersRecord gr = new GuttersRecord(r);
+        maxRowOutlineLevel = 
+          gr.getRowOutlineLevel() > 0 ? gr.getRowOutlineLevel() - 1 : 0;
+        maxColumnOutlineLevel = 
+          gr.getColumnOutlineLevel() > 0 ? gr.getRowOutlineLevel() - 1 : 0;
+      }
       else if (type == Type.BOF)
       {
         BOFRecord br = new BOFRecord(r);
@@ -1591,8 +1639,10 @@ final class SheetReader
                                     workbook.getDrawingGroup(),
                                     workbookSettings);
 
+      // Sometimes Excel writes out Continue records instead of drawing
+      // records, so forcibly hack all of these into a drawing record
       Record r2 = excelFile.next();
-      if (r2.getType() == Type.MSODRAWING)
+      if (r2.getType() == Type.MSODRAWING || r2.getType() == Type.CONTINUE)
       {
         MsoDrawingRecord mso = new MsoDrawingRecord(r2);
         comment.addMso(mso);
@@ -1776,4 +1826,24 @@ final class SheetReader
 
     outOfBoundsCells.clear();
   }
+
+  /** 
+   * Accessor for the maximum column outline level
+   *
+   * @return the maximum column outline level, or 0 if no outlines/groups
+   */
+  public int getMaxColumnOutlineLevel() 
+  {
+    return maxColumnOutlineLevel;
+  }
+
+  /** 
+   * Accessor for the maximum row outline level
+   *
+   * @return the maximum row outline level, or 0 if no outlines/groups
+   */
+  public int getMaxRowOutlineLevel() 
+  {
+    return maxRowOutlineLevel;
+  }
 }
