@@ -680,7 +680,7 @@ public class WorkbookParser extends Workbook
 
         if (bof.isBiff8())
         {
-          br = new BoundsheetRecord(r);
+          br = new BoundsheetRecord(r, settings);
         }
         else
         {
@@ -868,7 +868,8 @@ public class WorkbookParser extends Workbook
       {
         logger.warn("Usage of a local non-builtin name");
       } 
-      else if (nr.getBuiltInName() == BuiltInName.PRINT_AREA)
+      else if (nr.getBuiltInName() == BuiltInName.PRINT_AREA || 
+               nr.getBuiltInName() == BuiltInName.PRINT_TITLES)
       {
         // appears to use the internal tab number rather than the
         // external sheet index
