@@ -21,6 +21,7 @@ package jxl.write.biff;
 
 import java.io.File;
 import java.net.URL;
+import java.net.MalformedURLException;
 import java.util.ArrayList;
 
 import common.Assert;
@@ -29,6 +30,7 @@ import common.Logger;
 import jxl.CellType;
 import jxl.Hyperlink;
 import jxl.Range;
+import jxl.WorkbookSettings;
 import jxl.biff.CellReferenceHelper;
 import jxl.biff.IntegerHelper;
 import jxl.biff.SheetRangeImpl;
@@ -36,6 +38,7 @@ import jxl.biff.StringHelper;
 import jxl.biff.Type;
 import jxl.biff.WritableRecordData;
 import jxl.write.Label;
+import jxl.write.WritableHyperlink;
 import jxl.write.WritableCell;
 import jxl.write.WritableSheet;
 
@@ -134,8 +137,21 @@ public class HyperlinkRecord extends WritableRecordData
   {
     super(Type.HLINK);
 
-    Assert.verify(h instanceof jxl.read.biff.HyperlinkRecord);
+    if (h instanceof jxl.read.biff.HyperlinkRecord)
+    {
+      copyReadHyperlink(h, s);
+    }
+    else
+    {
+      copyWritableHyperlink(h, s);
+    }
+  }
 
+  /**
+   * Copies a hyperlink read in from a read only sheet
+   */
+  private void copyReadHyperlink(Hyperlink h, WritableSheet s)
+  {
     jxl.read.biff.HyperlinkRecord hl = (jxl.read.biff.HyperlinkRecord) h;
 
     data = hl.getRecord().getData();
@@ -171,6 +187,50 @@ public class HyperlinkRecord extends WritableRecordData
     modified = false;
   }
 
+  /**
+   * Copies a hyperlink read in from a writable sheet.
+   * Used when copying writable sheets
+   *
+   * @param hl the hyperlink from the read spreadsheet
+   */
+  private void copyWritableHyperlink(Hyperlink hl, WritableSheet s)
+  {
+    HyperlinkRecord h = (HyperlinkRecord) hl;
+
+    firstRow = h.firstRow;
+    lastRow = h.lastRow;
+    firstColumn = h.firstColumn;
+    lastColumn = h.lastColumn;
+
+    if (h.url != null)
+    {
+      try
+      {
+        url = new URL(h.url.toString());
+      }
+      catch (MalformedURLException e)
+      {
+        // should never get a malformed url as a result url.toString()
+        Assert.verify(false);
+      }
+    }
+    
+    if (h.file != null)
+    {
+      file = new File(h.file.getPath());
+    }
+
+    location = h.location;
+    contents = h.contents;
+    linkType = h.linkType;
+    modified = true;
+
+    sheet = s;
+    range = new SheetRangeImpl(s, 
+                               firstColumn, firstRow,
+                               lastColumn, lastRow);
+  }
+
   /**
    * Constructs a URL hyperlink to a range of cells
    *
@@ -548,7 +608,6 @@ public class HyperlinkRecord extends WritableRecordData
       if (l.getString().equals(prevurlString) ||
           l.getString().equals(prevurlString2))
       {
-        logger.debug("equal");
         l.setString(url.toString());
       }
     }   
