@@ -107,6 +107,8 @@ class SheetCopier
   private boolean chartOnly;
   private ButtonPropertySetRecord buttonPropertySet;
   private int numRows;
+  private int maxRowOutlineLevel;
+  private int maxColumnOutlineLevel;
 
   // Objects used to maintain state during the copy process
   private HashMap xfRecords;
@@ -114,7 +116,6 @@ class SheetCopier
   private HashMap formats;
 
   public SheetCopier(Sheet f, WritableSheet t)
-
   {
     fromSheet = (SheetImpl) f;
     toSheet = (WritableSheetImpl) t;
@@ -255,6 +256,8 @@ class SheetCopier
         rr.setRowDetails(rowprops[i].getRowHeight(), 
                          rowprops[i].matchesDefaultFontHeight(),
                          rowprops[i].isCollapsed(),
+                         rowprops[i].getOutlineLevel(),
+                         rowprops[i].getGroupStart(),
                          format);
         numRows = Math.max(numRows, rowprops[i].getRowNumber() + 1);
       }
@@ -396,8 +399,119 @@ class SheetCopier
       buttonPropertySet = new ButtonPropertySetRecord
         (fromSheet.getButtonPropertySet());
     }
+
+    // Copy the outline levels
+    maxRowOutlineLevel = fromSheet.getMaxRowOutlineLevel();
+    maxColumnOutlineLevel = fromSheet.getMaxColumnOutlineLevel();
   }
 
+  /**
+   * Copies a sheet from a read-only version to the writable version.
+   * Performs shallow copies
+   */
+  public void copyWritableSheet()
+  {
+    shallowCopyCells();
+
+    /*
+    // Copy the column formats
+    Iterator cfit = fromWritableSheet.columnFormats.iterator();
+    while (cfit.hasNext())
+    {
+      ColumnInfoRecord cv = new ColumnInfoRecord
+        ((ColumnInfoRecord) cfit.next());
+      columnFormats.add(cv);
+    }
+
+    // Copy the merged cells
+    Range[] merged = fromWritableSheet.getMergedCells();
+
+    for (int i = 0; i < merged.length; i++)
+    {
+      mergedCells.add(new SheetRangeImpl((SheetRangeImpl)merged[i], this));
+    }
+
+    // Copy the row properties
+    try
+    {
+      RowRecord[] copyRows = fromWritableSheet.rows;
+      RowRecord row = null;
+      for (int i = 0; i < copyRows.length ; i++)
+      {
+        row = copyRows[i];
+        
+        if (row != null &&
+            (!row.isDefaultHeight() ||
+             row.isCollapsed()))
+        {
+          RowRecord rr = getRowRecord(i);
+          rr.setRowDetails(row.getRowHeight(), 
+                           row.matchesDefaultFontHeight(),
+                           row.isCollapsed(), 
+                           row.getStyle());
+        }
+      }
+    }
+    catch (RowsExceededException e)
+    {
+      // Handle the rows exceeded exception - this cannot occur since
+      // the sheet we are copying from will have a valid number of rows
+      Assert.verify(false);
+    }
+
+    // Copy the horizontal page breaks
+    rowBreaks = new ArrayList(fromWritableSheet.rowBreaks);
+
+    // Copy the vertical page breaks
+    columnBreaks = new ArrayList(fromWritableSheet.columnBreaks);
+
+    // Copy the data validations
+    DataValidation rdv = fromWritableSheet.dataValidation;
+    if (rdv != null)
+    {
+      dataValidation = new DataValidation(rdv, 
+                                          workbook,
+                                          workbook, 
+                                          workbookSettings);
+    }
+
+    // Copy the charts 
+    sheetWriter.setCharts(fromWritableSheet.getCharts());
+
+    // Copy the drawings
+    DrawingGroupObject[] dr = si.getDrawings();
+    for (int i = 0 ; i < dr.length ; i++)
+    {
+      if (dr[i] instanceof jxl.biff.drawing.Drawing)
+      {
+        WritableImage wi = new WritableImage(dr[i], 
+                                             workbook.getDrawingGroup());
+        drawings.add(wi);
+        images.add(wi);
+      }
+
+      // Not necessary to copy the comments, as they will be handled by
+      // the deep copy of the individual cells
+    }
+
+    // Copy the workspace options
+    sheetWriter.setWorkspaceOptions(fromWritableSheet.getWorkspaceOptions());
+
+    // Copy the environment specific print record
+    if (fromWritableSheet.plsRecord != null)
+    {
+      plsRecord = new PLSRecord(fromWritableSheet.plsRecord);
+    }
+
+    // Copy the button property set
+    if (fromWritableSheet.buttonPropertySet != null)
+    {
+      buttonPropertySet = new ButtonPropertySetRecord
+        (fromWritableSheet.buttonPropertySet);
+    }
+    */
+  }  
+
   /**
    * Imports a sheet from a different workbook, doing a deep copy
    */
@@ -476,6 +590,8 @@ class SheetCopier
         rr.setRowDetails(rowrec.getRowHeight(), 
                          rowrec.matchesDefaultFontHeight(),
                          rowrec.isCollapsed(),
+                         rowrec.getOutlineLevel(),
+                         rowrec.getGroupStart(),
                          format);
         numRows = Math.max(numRows, rowprops[i].getRowNumber() + 1);
       }
@@ -655,6 +771,10 @@ class SheetCopier
     }
 
     importNames();
+
+    // Copy the outline levels
+    maxRowOutlineLevel = fromSheet.getMaxRowOutlineLevel();
+    maxColumnOutlineLevel = fromSheet.getMaxColumnOutlineLevel();
   }
 
   /**
@@ -945,4 +1065,24 @@ class SheetCopier
   {
     return numRows;
   }
+
+  /** 
+   * Accessor for the maximum column outline level
+   *
+   * @return the maximum column outline level, or 0 if no outlines/groups
+   */
+  public int getMaxColumnOutlineLevel() 
+  {
+    return maxColumnOutlineLevel;
+  }
+
+  /** 
+   * Accessor for the maximum row outline level
+   *
+   * @return the maximum row outline level, or 0 if no outlines/groups
+   */
+  public int getMaxRowOutlineLevel() 
+  {
+    return maxRowOutlineLevel;
+  }
 }
