@@ -164,6 +164,16 @@ final class SheetWriter
    */
   private boolean chartOnly;
 
+  /**
+   * The maximum row outline level
+   */
+  private int maxRowOutlineLevel;
+
+  /**
+   * The maximum column outline level
+   */
+  private int maxColumnOutlineLevel;
+
   /**
    * A handle back to the writable sheet, in order for this class
    * to invoke the get accessor methods
@@ -263,6 +273,9 @@ final class SheetWriter
     outputFile.write(gsr);
 
     GuttersRecord gutr = new GuttersRecord();
+    gutr.setMaxColumnOutline(maxColumnOutlineLevel + 1);
+    gutr.setMaxRowOutline(maxRowOutlineLevel + 1);
+
     outputFile.write(gutr);
 
     DefaultRowHeightRecord drhr = new DefaultRowHeightRecord
@@ -271,6 +284,16 @@ final class SheetWriter
                 SheetSettings.DEFAULT_DEFAULT_ROW_HEIGHT);
     outputFile.write(drhr);
 
+    if (maxRowOutlineLevel > 0)
+    {
+      workspaceOptions.setRowOutlines(true);
+    }
+
+    if (maxColumnOutlineLevel > 0)
+    {
+      workspaceOptions.setColumnOutlines(true);
+    }
+
     workspaceOptions.setFitToPages(settings.getFitToPages());
     outputFile.write(workspaceOptions);
 
@@ -606,7 +629,9 @@ final class SheetWriter
                     ArrayList   cb,
                     ArrayList   hl,
                     MergedCells mc,
-                    TreeSet     cf)
+                    TreeSet     cf,
+                    int         mrol,
+                    int         mcol)
   {
     rows = rws;
     rowBreaks = rb;
@@ -614,6 +639,8 @@ final class SheetWriter
     hyperlinks = hl;
     mergedCells = mc;
     columnFormats = cf;
+    maxRowOutlineLevel = mrol;
+    maxColumnOutlineLevel = mcol;
   }
 
   /**
