@@ -1,4 +1,4 @@
-/*********************************************************************
+/**********************************************************************
 *
 *      Copyright (C) 2002 Andrew Khan
 *
@@ -69,6 +69,7 @@ import jxl.write.DateTime;
 import jxl.write.Label;
 import jxl.write.Number;
 import jxl.write.WritableCell;
+import jxl.write.WritableCellFormat;
 import jxl.write.WritableFont;
 import jxl.write.WritableHyperlink;
 import jxl.write.WritableImage;
@@ -205,6 +206,16 @@ class WritableSheetImpl implements WritableSheet
    */
   private boolean drawingsModified;
 
+  /**
+   * The maximum row outline level
+   */
+  private int maxRowOutlineLevel;
+
+  /**
+   * The maximum column outline level
+   */
+  private int maxColumnOutlineLevel;
+
   /**
    * The settings for this sheet
    */
@@ -904,6 +915,12 @@ class WritableSheetImpl implements WritableSheet
   {
     if (row < 0 || row >= numRows)
     {
+      // Call rowRemoved anyway, to adjust the named cells
+      if (workbookSettings.getFormulaAdjust())
+      {
+        workbook.rowRemoved(this, row);
+      }
+
       return;
     }
 
@@ -1085,7 +1102,7 @@ class WritableSheetImpl implements WritableSheet
 
     if (rowrec == null)
     {
-      rowrec = new RowRecord(row);
+      rowrec = new RowRecord(row, this);
       rows[row] = rowrec;
     }
 
@@ -1365,7 +1382,11 @@ class WritableSheetImpl implements WritableSheet
       xfr = null;
     }
 
-    rowrec.setRowDetails(view.getSize(), false, view.isHidden(),
+    rowrec.setRowDetails(view.getSize(),
+                         false,
+                         view.isHidden(),
+                         0,
+                         false,
                          xfr);
     numRows = Math.max(numRows, row + 1);
   }
@@ -1395,7 +1416,9 @@ class WritableSheetImpl implements WritableSheet
                              columnBreaks,
                              hyperlinks, 
                              mergedCells, 
-                             columnFormats );
+                             columnFormats,
+                             maxRowOutlineLevel,
+                             maxColumnOutlineLevel);
     sheetWriter.setDimensions(getRows(), getColumns());
     sheetWriter.setSettings(settings);
     sheetWriter.setPLS(plsRecord);
@@ -1408,139 +1431,6 @@ class WritableSheetImpl implements WritableSheet
     sheetWriter.write();
   }
 
-  /** 
-   * Copies the cell contents from the specified sheet into this one
-   *
-   * @param s the sheet to copy
-   */
-  private void copyCells(Sheet s)
-  {
-    // Copy the cells
-    int cells = s.getRows();
-    Cell[] row = null;
-    Cell cell = null;
-    for (int i = 0;  i < cells; i++)
-    {
-      row = s.getRow(i);
-
-      for (int j = 0; j < row.length; j++)
-      {
-        cell = row[j];
-        CellType ct = cell.getType();
-
-        // Encase the calls to addCell in a try-catch block
-        // These should not generate any errors, because we are
-        // copying from an existing spreadsheet.  In the event of
-        // errors, catch the exception and then bomb out with an
-        // assertion
-        try
-        {
-          if (ct == CellType.LABEL)
-          {
-            Label l = new Label((LabelCell) cell);
-            addCell(l);
-          }
-          else if (ct == CellType.NUMBER)
-          {
-            Number n = new Number((NumberCell) cell);
-            addCell(n);
-          }
-          else if (ct == CellType.DATE)
-          {
-            DateTime dt = new DateTime((DateCell) cell);
-            addCell(dt);
-          }
-          else if (ct == CellType.BOOLEAN)
-          {
-            Boolean b = new Boolean((BooleanCell) cell);
-            addCell(b);
-          }
-          else if (ct == CellType.NUMBER_FORMULA)
-          {
-            ReadNumberFormulaRecord fr = new ReadNumberFormulaRecord
-              ((FormulaData) cell);
-            addCell(fr);
-          }
-          else if (ct == CellType.STRING_FORMULA)
-          {
-            ReadStringFormulaRecord fr = new ReadStringFormulaRecord
-              ((FormulaData) cell);
-            addCell(fr);
-          }
-          else if( ct == CellType.BOOLEAN_FORMULA)
-          {
-            ReadBooleanFormulaRecord fr = new ReadBooleanFormulaRecord
-              ((FormulaData) cell);
-            addCell(fr);
-          }
-          else if (ct == CellType.DATE_FORMULA)
-          {
-            ReadDateFormulaRecord fr = new ReadDateFormulaRecord
-              ((FormulaData) cell);
-            addCell(fr);
-          }
-          else if(ct == CellType.FORMULA_ERROR)
-          {
-            ReadErrorFormulaRecord fr = new ReadErrorFormulaRecord
-              ((FormulaData) cell);
-            addCell(fr);
-          }
-          else if (ct == CellType.EMPTY)
-          {
-            if (cell.getCellFormat() != null)
-            {
-              // It is a blank cell, rather than an empty cell, so
-              // it may have formatting information, so
-              // it must be copied
-              Blank b = new Blank(cell);
-              addCell(b);
-            }
-          }
-        }
-        catch (WriteException e)
-        {
-          Assert.verify(false);
-        }
-      }
-    }
-  }
-
-  /** 
-   * Copies the cell contents from the specified sheet into this one.  This
-   * differs from the previous method in that it uses deep copies to copy
-   * each cell
-   * Used for creating duplicates of WritableSheets
-   *
-   * @param s the sheet to copy
-   */
-  private void copyCells(WritableSheet s)
-  {
-    // Copy the cells
-    int cells = s.getRows();
-    Cell[] row = null;
-    Cell cell = null;
-    for (int i = 0;  i < cells; i++)
-    {
-      row = s.getRow(i);
-
-      for (int j = 0; j < row.length; j++)
-      {
-        cell = row[j];
-
-        try
-        {
-          WritableCell wc = ( (WritableCell) cell).copyTo(cell.getColumn(), 
-                                                          cell.getRow());
-          addCell(wc);
-        }
-        catch (WriteException e)
-        {
-          Assert.verify(false);
-        }
-      }
-    }
-  }
-
   /**
    * Copies the specified sheet, row by row and cell by cell
    * 
@@ -1572,6 +1462,8 @@ class WritableSheetImpl implements WritableSheet
     buttonPropertySet = si.getButtonPropertySet();
     numRows = si.getRows();
     autoFilter = si.getAutoFilter();
+    maxRowOutlineLevel = si.getMaxRowOutlineLevel();
+    maxColumnOutlineLevel = si.getMaxColumnOutlineLevel();
   }
 
   /**
@@ -1582,109 +1474,27 @@ class WritableSheetImpl implements WritableSheet
   void copy(WritableSheet s)
   {
     settings = new SheetSettings(s.getSettings(), this);
-
-    copyCells(s);
-
-    // Copy the column formats
-    Iterator cfit = ( (WritableSheetImpl) s).columnFormats.iterator();
-    while (cfit.hasNext())
-    {
-      ColumnInfoRecord cv = new ColumnInfoRecord
-        ((ColumnInfoRecord) cfit.next());
-      columnFormats.add(cv);
-    }
-
-    // Copy the merged cells
-    Range[] merged = s.getMergedCells();
-
-    for (int i = 0; i < merged.length; i++)
-    {
-      mergedCells.add(new SheetRangeImpl((SheetRangeImpl)merged[i], this));
-    }
-
-    // Copy the row properties
-    try
-    {
-      RowRecord[] copyRows = ( (WritableSheetImpl) s).rows;
-      RowRecord row = null;
-      for (int i = 0; i < copyRows.length ; i++)
-      {
-        row = copyRows[i];
-        
-        if (row != null &&
-            (!row.isDefaultHeight() ||
-             row.isCollapsed()))
-        {
-          RowRecord rr = getRowRecord(i);
-          rr.setRowDetails(row.getRowHeight(), 
-                           row.matchesDefaultFontHeight(),
-                           row.isCollapsed(), 
-                           row.getStyle());
-        }
-      }
-    }
-    catch (RowsExceededException e)
-    {
-      // Handle the rows exceeded exception - this cannot occur since
-      // the sheet we are copying from will have a valid number of rows
-      Assert.verify(false);
-    }
-
-    // Copy the headers and footers
     WritableSheetImpl si = (WritableSheetImpl) s;
 
-    //    sheetWriter.setHeader(si.getHeader());
-    //    sheetWriter.setFooter(si.getFooter());
+    WritableSheetCopier sc = new WritableSheetCopier(s, this);
+    sc.setColumnFormats(si.columnFormats, columnFormats);
+    sc.setMergedCells(si.mergedCells, mergedCells);
+    sc.setRows(si.rows);
+    sc.setRowBreaks(si.rowBreaks, rowBreaks);
+    sc.setColumnBreaks(si.columnBreaks, columnBreaks);
+    sc.setDataValidation(si.dataValidation);
+    sc.setSheetWriter(sheetWriter);
+    sc.setDrawings(si.drawings, drawings, images);
+    sc.setWorkspaceOptions(si.getWorkspaceOptions());
+    sc.setPLSRecord(si.plsRecord);
+    sc.setButtonPropertySetRecord(si.buttonPropertySet);
+    sc.setHyperlinks(si.hyperlinks, hyperlinks);
 
-    // Copy the horizontal page breaks
-    rowBreaks = new ArrayList(si.rowBreaks);
+    sc.copySheet();
 
-    // Copy the vertical page breaks
-    columnBreaks = new ArrayList(si.columnBreaks);
-
-    // Copy the data validations
-    DataValidation rdv = si.dataValidation;
-    if (rdv != null)
-    {
-      dataValidation = new DataValidation(rdv, 
-                                          workbook,
-                                          workbook, 
-                                          workbookSettings);
-    }
-
-    // Copy the charts 
-    sheetWriter.setCharts(si.getCharts());
-
-    // Copy the drawings
-    DrawingGroupObject[] dr = si.getDrawings();
-    for (int i = 0 ; i < dr.length ; i++)
-    {
-      if (dr[i] instanceof jxl.biff.drawing.Drawing)
-      {
-        WritableImage wi = new WritableImage(dr[i], 
-                                             workbook.getDrawingGroup());
-        drawings.add(wi);
-        images.add(wi);
-      }
-
-      // Not necessary to copy the comments, as they will be handled by
-      // the deep copy of the individual cells
-    }
-
-    // Copy the workspace options
-    sheetWriter.setWorkspaceOptions(si.getWorkspaceOptions());
-
-    // Copy the environment specific print record
-    if (si.plsRecord != null)
-    {
-      plsRecord = new PLSRecord(si.plsRecord);
-    }
-
-    // Copy the button property set
-    if (si.buttonPropertySet != null)
-    {
-      buttonPropertySet = new ButtonPropertySetRecord(si.buttonPropertySet);
-    }
+    dataValidation = sc.getDataValidation();
+    plsRecord = sc.getPLSRecord();
+    buttonPropertySet = sc.getButtonPropertySet();
   }
 
   /**
@@ -1849,11 +1659,17 @@ class WritableSheetImpl implements WritableSheet
       contents = ( (HyperlinkRecord) h).getContents();
     }
 
+    // If the cell type is a label, then preserve the cell contents
+    // and most of the format (apart from the font)
+    // otherwise overwrite the cell content and the format with the contents
+    // and the standard hyperlink format
     if (c.getType() == CellType.LABEL)
     {
       Label l = (Label) c;
       l.setString(contents);
-      l.setCellFormat(WritableWorkbook.HYPERLINK_STYLE);
+      WritableCellFormat wcf = new WritableCellFormat(l.getCellFormat());
+      ( (XFRecord) wcf).setFont(WritableWorkbook.HYPERLINK_FONT);
+      l.setCellFormat(wcf);
     }
     else
     {
@@ -1870,7 +1686,7 @@ class WritableSheetImpl implements WritableSheet
         if (i != h.getRow() && j != h.getColumn())
         {
           // Set the cell to be empty
-          if (rows[i] != null)
+          if (rows.length < h.getLastColumn() && rows[i] != null)
           {
             rows[i].removeCell(j);
           }
@@ -1916,6 +1732,145 @@ class WritableSheetImpl implements WritableSheet
     return range;
   }
 
+  /** 
+   * Sets a row grouping
+   *
+   * @param row1 the first row of the group
+   * @param row2 the last row of the group
+   * @param collapsed should the group be collapsed?
+   * @exception WriteException
+   * @exception RowsExceededException
+   */
+  public void setRowGroup(int row1, int row2, 
+                          boolean collapsed) 
+    throws WriteException, RowsExceededException 
+  {
+    if (row2 < row1)
+    {
+      logger.warn("Cannot merge cells - top and bottom rows incorrectly " + 
+                  "specified");
+    }
+
+    for (int i = row1; i <= row2; i++) 
+    {
+      RowRecord row = getRowRecord(i);
+      numRows = Math.max(i+1, numRows);
+      row.incrementOutlineLevel();
+      row.setCollapsed(collapsed);
+      maxRowOutlineLevel = Math.max(maxRowOutlineLevel, 
+                                    row.getOutlineLevel());
+    }
+  }
+
+  /** 
+   * Unsets a row grouping
+   *
+   * @param row1 the first row to unset
+   * @param row2 the last row to unset
+   * @exception WriteException
+   * @exception RowsExceededException
+   */
+  public void unsetRowGroup(int row1, int row2) 
+    throws WriteException, RowsExceededException 
+  {
+    if (row2 < row1)
+    {
+      logger.warn("Cannot merge cells - top and bottom rows incorrectly " +
+                  "specified");
+    }
+
+    // Make sure the spreadsheet is up to size
+    if (row2 >= numRows)
+    {
+      logger.warn("" + row2 + 
+                  " is greater than the sheet bounds");
+      row2 = numRows - 1;
+    }
+
+    for (int i = row1; i <= row2; i++) 
+    {
+      rows[i].decrementOutlineLevel();
+    }
+
+    // Recalculate the max outline level
+    maxRowOutlineLevel = 0;
+    for (int i = rows.length; i-- > 0; ) 
+    {
+      maxRowOutlineLevel = Math.max(maxRowOutlineLevel, 
+                                    rows[i].getOutlineLevel());
+    }
+  }
+
+  /** 
+   * Sets a column grouping
+   *
+   * @param col1 the first column of the group
+   * @param col2 the last column of the group
+   * @param collapsed should the group be collapsed?
+   * @exception WriteException
+   * @exception RowsExceededException
+   */
+  public void setColumnGroup(int col1, int col2, boolean collapsed) 
+    throws WriteException, RowsExceededException 
+  {
+    if (col2 < col1)
+    {
+      logger.warn("Cannot merge cells - top and bottom rows incorrectly " +
+                  "specified");
+    }
+
+    for (int i = col1; i <= col2; i++) 
+    {
+      ColumnInfoRecord cir = getColumnInfo(i);
+
+      // Create the column info record if not present using a default
+      // cell view
+      if (cir == null)
+      {
+        setColumnView(i, new CellView());
+        cir = getColumnInfo(i);
+      }
+
+      cir.incrementOutlineLevel();
+      cir.setCollapsed(collapsed);
+      maxColumnOutlineLevel = Math.max(maxColumnOutlineLevel, 
+                                       cir.getOutlineLevel());
+    }
+  }
+
+  /** 
+   * Unsets a column grouping
+   *
+   * @param col1 the first column to unset
+   * @param col2 the last column to unset
+   * @exception WriteException
+   * @exception RowsExceededException
+   */
+  public void unsetColumnGroup(int col1, int col2) 
+    throws WriteException, RowsExceededException 
+  {
+    if (col2 < col1)
+    {
+      logger.warn("Cannot merge cells - top and bottom rows incorrectly " +
+                  "specified");
+    }
+
+    for (int i = col1; i <= col2; i++) 
+    {
+      ColumnInfoRecord cir = getColumnInfo(i);
+      cir.decrementOutlineLevel();
+    }
+    
+    // Recalculate the max outline level
+    maxColumnOutlineLevel = 0;
+    for (Iterator it = columnFormats.iterator(); it.hasNext(); ) 
+    {
+      ColumnInfoRecord cir = (ColumnInfoRecord)it.next();
+      maxColumnOutlineLevel = Math.max(maxColumnOutlineLevel, 
+                                       cir.getOutlineLevel());
+    }
+  }
+
   /**
    * Unmerges the specified cells.  The Range passed in should be one that
    * has been previously returned as a result of the getMergedCells method
@@ -2078,7 +2033,7 @@ class WritableSheetImpl implements WritableSheet
    *
    * @return the charts on this sheet
    */
-  private Chart[] getCharts()
+  Chart[] getCharts()
   {
     return sheetWriter.getCharts();
   }
@@ -2108,7 +2063,9 @@ class WritableSheetImpl implements WritableSheet
                              columnBreaks,
                              hyperlinks, 
                              mergedCells, 
-                             columnFormats );
+                             columnFormats,
+                             maxRowOutlineLevel,
+                             maxColumnOutlineLevel);
     sheetWriter.setDimensions(getRows(), getColumns());
     sheetWriter.checkMergedBorders();
   }
@@ -2436,6 +2393,17 @@ class WritableSheetImpl implements WritableSheet
     Assert.verify(newSize == origSize -1);
   }
 
+  /**
+   * Removes the data validation for the specified cell.  Called from
+   * CellValue in response to a cell being replaced
+   *
+   * @param cv the cell being removed
+   */
+  void removeDataValidation(CellValue cv)
+  {
+    dataValidation.removeDataValidation(cv.getColumn(), cv.getRow());
+  }
+
   /**
    * Accessor for the page breaks on this sheet
    *
@@ -2592,5 +2560,7 @@ class WritableSheetImpl implements WritableSheet
     chartOnly = si.isChartOnly();
     buttonPropertySet = si.getButtonPropertySet();
     numRows = si.getRows();
+    maxRowOutlineLevel = si.getMaxRowOutlineLevel();
+    maxColumnOutlineLevel = si.getMaxColumnOutlineLevel();
   }
 }
