@@ -154,6 +154,8 @@ public class WritableWorkbookImpl extends WritableWorkbook
    */
   private CountryRecord countryRecord;
 
+  // synchronizer object for static unitiatialization
+  private static Object SYNCHRONIZER = new Object();
   /**
    * The names of any add in functions
    */
@@ -183,13 +185,20 @@ public class WritableWorkbookImpl extends WritableWorkbook
     rcirCells = new ArrayList();
     styles = new Styles();
 
-    // Reset the statically declared styles.  Thanks to Brendan for this
-    WritableWorkbook.ARIAL_10_PT.uninitialize();
-    WritableWorkbook.HYPERLINK_FONT.uninitialize();
-    WritableWorkbook.NORMAL_STYLE.uninitialize();
-    WritableWorkbook.HYPERLINK_STYLE.uninitialize();
-    WritableWorkbook.HIDDEN_STYLE.uninitialize();
-    DateRecord.defaultDateFormat.uninitialize();
+    // Reset the statically declared styles.  These are no longer needed
+    // because the Styles class will intercept all calls within 
+    // CellValue.setCellDetails and if it detects a standard format, then it
+    // will return a clone.  In short, the static cell values will
+    // never get initialized anyway.  Still, just to be extra sure...
+    synchronized(SYNCHRONIZER)
+    {
+      WritableWorkbook.ARIAL_10_PT.uninitialize();
+      WritableWorkbook.HYPERLINK_FONT.uninitialize();
+      WritableWorkbook.NORMAL_STYLE.uninitialize();
+      WritableWorkbook.HYPERLINK_STYLE.uninitialize();
+      WritableWorkbook.HIDDEN_STYLE.uninitialize();
+      DateRecord.defaultDateFormat.uninitialize();
+     }
 
     WritableFonts wf = new WritableFonts(this);
     fonts = wf;
@@ -217,13 +226,20 @@ public class WritableWorkbookImpl extends WritableWorkbook
     super();
     WorkbookParser wp = (WorkbookParser) w;
 
-    // Reset the statically declared styles.  Thanks to Brendan for this
-    WritableWorkbook.ARIAL_10_PT.uninitialize();
-    WritableWorkbook.HYPERLINK_FONT.uninitialize();
-    WritableWorkbook.NORMAL_STYLE.uninitialize();
-    WritableWorkbook.HYPERLINK_STYLE.uninitialize();
-    WritableWorkbook.HIDDEN_STYLE.uninitialize();
-    DateRecord.defaultDateFormat.uninitialize();
+    // Reset the statically declared styles.  These are no longer needed
+    // because the Styles class will intercept all calls within 
+    // CellValue.setCellDetails and if it detects a standard format, then it
+    // will return a clone.  In short, the static cell values will
+    // never get initialized anyway.  Still, just to be extra sure...
+    synchronized(SYNCHRONIZER)
+    {
+      WritableWorkbook.ARIAL_10_PT.uninitialize();
+      WritableWorkbook.HYPERLINK_FONT.uninitialize();
+      WritableWorkbook.NORMAL_STYLE.uninitialize();
+      WritableWorkbook.HYPERLINK_STYLE.uninitialize();
+      WritableWorkbook.HIDDEN_STYLE.uninitialize();
+      DateRecord.defaultDateFormat.uninitialize();
+     }
 
     closeStream = cs;
     sheets = new ArrayList();
@@ -628,8 +644,48 @@ public class WritableWorkbookImpl extends WritableWorkbook
                     range.getBottomRight().getRow(),
                     false);
       }
+      
+      // Check to see if print titles by row were set
+      Range rangeR = wsi.getSettings().getPrintTitlesRow();
+      Range rangeC = wsi.getSettings().getPrintTitlesCol();      
+      if (rangeR != null && rangeC != null)
+      {
+        addNameArea(BuiltInName.PRINT_TITLES, 
+                    wsi,
+                    rangeR.getTopLeft().getColumn(),
+                    rangeR.getTopLeft().getRow(),
+                    rangeR.getBottomRight().getColumn(),
+                    rangeR.getBottomRight().getRow(),
+                    rangeC.getTopLeft().getColumn(),
+                    rangeC.getTopLeft().getRow(),
+                    rangeC.getBottomRight().getColumn(),
+                    rangeC.getBottomRight().getRow(),
+                    false);  
+      }
+      // Check to see if print titles by row were set
+      else if (rangeR != null)
+      {
+    	  addNameArea(BuiltInName.PRINT_TITLES, 
+                    wsi,
+                    rangeR.getTopLeft().getColumn(),
+                    rangeR.getTopLeft().getRow(),
+                    rangeR.getBottomRight().getColumn(),
+                    rangeR.getBottomRight().getRow(),
+                    false);
+      }
+      // Check to see if print titles by column were set 
+      else if (rangeC != null)
+      {
+        addNameArea(BuiltInName.PRINT_TITLES, 
+                    wsi,
+                    rangeC.getTopLeft().getColumn(),
+                    rangeC.getTopLeft().getRow(),
+                    rangeC.getBottomRight().getColumn(),
+                    rangeC.getBottomRight().getRow(), 
+                    false);
+      }
     }
-
+    
     // Rationalize all the XF and number formats
     if (!settings.getRationalizationDisabled())
     {
@@ -691,7 +747,29 @@ public class WritableWorkbookImpl extends WritableWorkbook
     Prot4RevPassRecord p4rp = new Prot4RevPassRecord();
     outputFile.write(p4rp);
 
-    Window1Record w1r = new Window1Record();
+    // If no sheet is identified as being selected, then select
+    // the first one
+    boolean sheetSelected = false;
+    WritableSheetImpl wsheet = null;
+    int selectedSheetIndex = 0;
+    for (int i = 0 ; i < getNumberOfSheets() && !sheetSelected ; i++)
+    {
+      wsheet = (WritableSheetImpl) getSheet(i);
+      if (wsheet.getSettings().isSelected())
+      {
+        sheetSelected = true;
+        selectedSheetIndex = i;
+      }
+    }
+
+    if (!sheetSelected)
+    {
+      wsheet = (WritableSheetImpl) getSheet(0);
+      wsheet.getSettings().setSelected(true);
+      selectedSheetIndex = 0;
+    }
+
+    Window1Record w1r = new Window1Record(selectedSheetIndex);
     outputFile.write(w1r);
 
     BackupRecord bkr = new BackupRecord(false);
@@ -737,8 +815,7 @@ public class WritableWorkbookImpl extends WritableWorkbook
     {
       boundsheetPos[i] = outputFile.getPos();
       sheet = getSheet(i);
-      BoundsheetRecord br = new BoundsheetRecord
-        (sheet.getName());
+      BoundsheetRecord br = new BoundsheetRecord(sheet.getName());
       if (sheet.getSettings().isHidden())
       {
         br.setHidden();
@@ -824,24 +901,6 @@ public class WritableWorkbookImpl extends WritableWorkbook
     EOFRecord eof = new EOFRecord();
     outputFile.write(eof);
 
-    // If no sheet is identified as being selected, then select
-    // the first one
-    boolean sheetSelected = false;
-    WritableSheetImpl wsheet = null;
-    for (int i = 0 ; i < getNumberOfSheets() && !sheetSelected ; i++)
-    {
-      wsheet = (WritableSheetImpl) getSheet(i);
-      if (wsheet.getSettings().isSelected())
-      {
-        sheetSelected = true;
-      }
-    }
-
-    if (!sheetSelected)
-    {
-      wsheet = (WritableSheetImpl) getSheet(0);
-      wsheet.getSettings().setSelected(true);
-    }
 
     // Write out the sheets
     for (int i = 0; i < getNumberOfSheets(); i++)
@@ -1283,6 +1342,16 @@ public class WritableWorkbookImpl extends WritableWorkbook
       CellValue cv = (CellValue) i.next();
       cv.columnInserted(s, externalSheetIndex, col);
     }
+
+    // Adjust any named cells
+    if (names != null)
+    {
+      for (Iterator i = names.iterator(); i.hasNext() ;)
+      {
+        NameRecord nameRecord = (NameRecord) i.next();
+        nameRecord.columnInserted(externalSheetIndex, col);
+      }
+    }
   }
 
   /**
@@ -1300,6 +1369,32 @@ public class WritableWorkbookImpl extends WritableWorkbook
       CellValue cv = (CellValue) i.next();
       cv.columnRemoved(s, externalSheetIndex, col);
     }
+
+    // Adjust any named cells
+    ArrayList removedNames = new ArrayList();
+    if (names != null)
+    {
+      for (Iterator i = names.iterator(); i.hasNext() ;)
+      {
+        NameRecord nameRecord = (NameRecord) i.next();
+        boolean removeName = nameRecord.columnRemoved(externalSheetIndex,
+                                                      col);
+
+        if (removeName)
+        {
+          removedNames.add(nameRecord);
+        }
+      }
+
+      // Remove any names which have been deleted
+      for (Iterator i = removedNames.iterator(); i.hasNext() ;)
+      {
+        NameRecord nameRecord = (NameRecord) i.next();
+        boolean removed = names.remove(nameRecord);
+        Assert.verify(removed, "Could not remove name " + 
+                      nameRecord.getName());
+      }
+    }
   }
 
   /**
@@ -1312,11 +1407,23 @@ public class WritableWorkbookImpl extends WritableWorkbook
   void rowInserted(WritableSheetImpl s, int row)
   {
     int externalSheetIndex = getExternalSheetIndex(s.getName());
+    
+    // Adjust the row infos
     for (Iterator i = rcirCells.iterator() ; i.hasNext() ;)
     {
       CellValue cv = (CellValue) i.next();
       cv.rowInserted(s, externalSheetIndex, row);
     }
+
+    // Adjust any named cells
+    if (names != null)
+    {
+      for (Iterator i = names.iterator(); i.hasNext() ;)
+      {
+        NameRecord nameRecord = (NameRecord) i.next();
+        nameRecord.rowInserted(externalSheetIndex, row);
+      }
+    }
   }
 
   /**
@@ -1334,6 +1441,31 @@ public class WritableWorkbookImpl extends WritableWorkbook
       CellValue cv = (CellValue) i.next();
       cv.rowRemoved(s, externalSheetIndex, row);
     }
+
+    // Adjust any named cells
+    ArrayList removedNames = new ArrayList();
+    if (names != null)
+    {
+      for (Iterator i = names.iterator(); i.hasNext() ;)
+      {
+        NameRecord nameRecord = (NameRecord) i.next();
+        boolean removeName = nameRecord.rowRemoved(externalSheetIndex, row);
+
+        if (removeName)
+        {
+          removedNames.add(nameRecord);
+        }
+      }
+
+      // Remove any names which have been deleted
+      for (Iterator i = removedNames.iterator(); i.hasNext() ;)
+      {
+        NameRecord nameRecord = (NameRecord) i.next();
+        boolean removed = names.remove(nameRecord);
+        Assert.verify(removed, "Could not remove name " + 
+                      nameRecord.getName());
+      }
+    }
   }
 
   /**
@@ -1505,9 +1637,16 @@ public class WritableWorkbookImpl extends WritableWorkbook
       }
     }
 
+    // Remove the name from the list of names and the associated hashmap
+    // of names (used to retrieve the name index).  If the name cannot
+    // be found, a warning is displayed
     if (found)
     {
       names.remove(pos);
+      if (nameRecords.remove(name) == null)
+      {
+        logger.warn("Could not remove " + name + " from index lookups");
+      }
     }
   }
 
@@ -1628,6 +1767,60 @@ public class WritableWorkbookImpl extends WritableWorkbook
     nameRecords.put(name, nr);
   }
 
+  /**
+   * Add new named area to this workbook with the given information.
+   * 
+   * @param name name to be created.
+   * @param sheet sheet containing the name
+   * @param firstCol  first column this name refers to.
+   * @param firstRow  first row this name refers to.
+   * @param lastCol   last column this name refers to.
+   * @param lastRow   last row this name refers to.
+   * @param firstCol2 first column this name refers to.
+   * @param firstRow2 first row this name refers to.
+   * @param lastCol2  last column this name refers to.
+   * @param lastRow2  last row this name refers to.
+   * @param global   TRUE if this is a global name, FALSE if this is tied to 
+   *                 the sheet
+   */
+  void addNameArea(BuiltInName name, 
+                   WritableSheet sheet,
+                   int firstCol, 
+                   int firstRow,  
+                   int lastCol,
+                   int lastRow,
+                   int firstCol2, 
+                   int firstRow2,  
+                   int lastCol2,
+                   int lastRow2,
+                   boolean global)
+  {
+    if (names == null)
+    {
+      names = new ArrayList();
+    }
+
+    int index = getInternalSheetIndex(sheet.getName());
+    int externalSheetIndex = getExternalSheetIndex(sheet.getName());
+
+    // Create a new name record.
+    NameRecord nr = 
+      new NameRecord(name, 
+                     index,
+                     externalSheetIndex, 
+                     firstRow2, lastRow2, 
+                     firstCol2, lastCol2,
+                     firstRow, lastRow, 
+                     firstCol, lastCol, 
+                     global);
+    
+    // Add new name to name array.
+    names.add(nr);
+    
+    // Add new name to name hash table.
+    nameRecords.put(name, nr);
+  }  
+  
   /**
    * Accessor for the workbook settings
    */
